#include <cstdio>
#include <cmath>
#include <cstring>
#include <algorithm>

#include "File.h"
#include "Packet.h"
#include "Tools.h"
#include "IO.h"
#include "time.h"

#ifdef RELEASE
#define FANCY_GUI
#endif // release


using namespace std;


void TreatDataPacket(char * dataPointer,int dataSize,char ** treatedDataPointer,int * treatedDataSize)
{
    *treatedDataPointer = (char *) malloc(dataSize*2);
    if(*treatedDataPointer == NULL)
    {
        printf("Treating data content: error out of memory");
        exit(1);
    }


    int pos = 0;
    for(int i = 0 ; i < dataSize ; i++)
    {
        if(dataPointer[i] <= 0x1F)
        {
            (*treatedDataPointer)[pos] = 0x5C;
            pos++;
            (*treatedDataPointer)[pos] = dataPointer[i]+0x20;
        }
        else if(dataPointer[i] == 0x5C)
        {
            (*treatedDataPointer)[pos] = 0x5C;
            pos++;
            (*treatedDataPointer)[pos] = 0x5C;
        }
        else
        {
            (*treatedDataPointer)[pos] = dataPointer[i];
        }
        pos++;
    }

    *treatedDataSize = pos;

}


void SendFile(char * filename, int fileSize, char * filePointer, char * foldername, char * devicename, int mode)
{
    Packet sendPacket;

    printf("Sending file %do at %s/%s/%s\n",fileSize,devicename,foldername,filename);

    long startTime = getTime();


    char data[1024];
    int dataSize = 2 + 2 + 8 + 6*2 + 8 + 12 + 4;

    sendPacket.Type = 0x01;
    sendPacket.Subtype = 0x45;
    sendPacket.Extended = 1;

    ToAsciiByte(mode,&data[0],2); //OW
    ToAsciiByte(5,&data[2],2); //Data Type
    ToAsciiByte(fileSize,&data[4],8); // file Size

    if(foldername[0] != '\0')
    {
        ToAsciiByte(8,&data[12],2); //Size D1 DOSSIER
    }
    else
    {
        ToAsciiByte(0,&data[12],2); //Size D1 DOSSIER
    }

    ToAsciiByte(12,&data[14],2); //Size D2 FICHIER
    ToAsciiByte(0,&data[16],2); //Size D3
    ToAsciiByte(0,&data[18],2); //Size D4
    ToAsciiByte(4,&data[20],2); //Size D5 PERIPH
    ToAsciiByte(0,&data[22],2); //Size D6

    int offset = 0;

    if(foldername[0] != '\0')
    {
        for(int i = 0; i < 8 ; i++)
        {
            data[24+offset] = foldername[i];
            offset++;
        }
    }
    for(int i = 0; i < 12 ; i++)
    {
        data[24+offset] = filename[i];
        offset++;
    }
    for(int i = 0; i < 4 ; i++)
    {
        data[24+offset] = devicename[i];
        offset++;
    }


    char * treatedDataPointer;
    int treatedDataSize;

    TreatDataPacket(data,dataSize,&treatedDataPointer,&treatedDataSize);

    sendPacket.DataSize = treatedDataSize;
    sendPacket.DataPointer = treatedDataPointer;

    if(SendPacketAndAck(sendPacket))
    {
        printf("Error while sending the file request !\n");
        return;
    }


    int dataChunkSize = 256;

    char dataBuffer[dataChunkSize+4+4];

    Packet DataPacket(0x02,0x45,1,4+4+dataChunkSize,dataBuffer);

    int packetCount = ceil(fileSize/(float)dataChunkSize);

    int dataLeft = fileSize;

    int loadingBarSize = 20;

    printf("%d packets to send !\n",packetCount);


    float loadingBarProgressPerPacket = (loadingBarSize/(float)packetCount);

    for(int i = 0 ; i < packetCount ; i++)
    {
        int dataToSend = min(dataChunkSize,dataLeft);


        #ifdef FANCY_GUI
        for(int j = 0 ; j < 50 ; j++)
            printf("\n");
        for(int j = -1 ; j < loadingBarSize+1 ; j++)
        printf("-");
        printf("\n-");
        for(int j = 0 ; j < loadingBarSize ; j++)
        {
            if(j < loadingBarProgressPerPacket*(i+1))
            {
                printf("#");
            }
            else
            {
                printf("|");
            }
        }
        printf("-\n");
        #else
        printf("Sending packet %d   %d/%d\n",(i+1),dataChunkSize*(i+1),fileSize);
        #endif






        memcpy(dataBuffer+8,filePointer + i*dataChunkSize,dataToSend);

        ToAsciiByte(packetCount,dataBuffer+0,4);
        ToAsciiByte(i+1,dataBuffer+4,4);


        char * treatedDataPointer;
        int treatedDataSize;

        TreatDataPacket(dataBuffer,dataToSend+4+4,&treatedDataPointer,&treatedDataSize);

        DataPacket.DataPointer = treatedDataPointer;
        DataPacket.DataSize = treatedDataSize;

        if(SendPacketAndAck(DataPacket))
        {
            printf("An error occured while sending file's data!\n");
            return;
        }

        dataLeft -= dataChunkSize;
    }

    int totalTime = getTime() -startTime;

    printf("Time to tranfere: %dms\n",totalTime);

}

