#include "Packet.h"
#include "Tools.h"
#include "IO.h"

#include <cstdlib>
#include <cstring>
#include <cstdio>


    Packet::Packet()
    {

    }

    Packet::Packet(int NewType, int NewSubtype, int NewExtended, int NewDataSize, char * NewDataPointer)
    {
        Type = NewType;
        Subtype = NewSubtype;
        Extended = NewExtended;
        DataSize = NewDataSize;
        DataPointer = NewDataPointer;
    }


    void Packet::Reset()
    {
        Type = 0;
        Subtype = 0;
        Extended = 0;
        DataSize = 0;
        DataPointer = NULL;
    }

    void Packet::ComputeChecksum(ComputedPacket PacketToSum)
    {
        unsigned char sum = 0;

        for(int i = 1 ; i < PacketToSum.PacketSize-2 ; i++)
        {
            sum += PacketToSum.PacketPointer[i];
        }


        sum = (~sum);

        sum += 1;


        Checksum = sum;
    }



    ComputedPacket Packet::PackPacket()
    {
        ComputedPacket GeneratedPacket;

        GeneratedPacket.PacketSize = 1+2+1+(Extended ? DataSize + 4 : 0)+2;

        GeneratedPacket.PacketPointer = (char *)malloc(GeneratedPacket.PacketSize);

        if(GeneratedPacket.PacketPointer == NULL)
        {
            error("Out of memory for a new packet !");
        }

        GeneratedPacket.PacketPointer[0] = Type;
        ToAsciiByte(Subtype,&GeneratedPacket.PacketPointer[1],2);
        ToAsciiByte(Extended,&GeneratedPacket.PacketPointer[3],1);

        int offset = 0;
        if(Extended)
        {
            offset = 4+DataSize;
            ToAsciiByte(DataSize,&GeneratedPacket.PacketPointer[4],4);
            memcpy(GeneratedPacket.PacketPointer+8,DataPointer,DataSize);
        }

        ComputeChecksum(GeneratedPacket);

        ToAsciiByte(Checksum,&GeneratedPacket.PacketPointer[4+offset],2);

        return GeneratedPacket;
    }

    int Packet::UnpackPacket(ComputedPacket ReceivedPacket)
    {
        int receivedChecksum;
        FromAsciiByte(&receivedChecksum,&ReceivedPacket.PacketPointer[ReceivedPacket.PacketSize-2],2);


        ComputeChecksum(ReceivedPacket);


        if(receivedChecksum != Checksum)
        {

            printf("Broken Packet !(%.2x|%.2x) %x %c%c %c ",receivedChecksum,Checksum,
                                                            ReceivedPacket.PacketPointer[0],

                                                            ReceivedPacket.PacketPointer[1],
                                                            ReceivedPacket.PacketPointer[2],

                                                            ReceivedPacket.PacketPointer[3]);

            if(ReceivedPacket.PacketPointer[3] == 31)
            {
                printf("%c%c%c%c [DATA]", ReceivedPacket.PacketPointer[4],
                                          ReceivedPacket.PacketPointer[5],
                                          ReceivedPacket.PacketPointer[6],
                                          ReceivedPacket.PacketPointer[7]);
            }

            return 1;

        }

        Type = ReceivedPacket.PacketPointer[0];
        FromAsciiByte(&Subtype,&ReceivedPacket.PacketPointer[1],2);
        FromAsciiByte(&Extended,&ReceivedPacket.PacketPointer[3],1);
        if(Extended)
        {
            FromAsciiByte(&DataSize,&ReceivedPacket.PacketPointer[4],4);
            DataPointer = &ReceivedPacket.PacketPointer[8];
        }



        return 0;
    }
